/*
 *	@name		gridftp.h
 *	@author		Christian Loeschen, TU Dresden
 *	@license	BSD
 */

#include "globus_ftp_client.h"

/*
 *	Define GRIDFTP_IRODS_MODULE, when the gridftp_client is built as iRODS module.
 *	Otherwise it's built as standalone gridftp client.
 */
#define GRIDFTP_IRODS_MODULE

/*
 *	Define GRIDFTP_DEBUG, when built for testing/debugging purposes.
 */
#define GRIDFTP_DEBUG

/*
 *	Define GRIDFTP_DISABLE_FEATURE_CHECK to disable feature check. Then
 *	gridftp_feat_check always returns TRUE for feature support.
 */
//#define GRIDFTP_DISABLE_FEATURE_CHECK

#ifndef GRIDFTP_CLIENT_H_
#define GRIDFTP_CLIENT_H_

#ifdef GRIDFTP_IRODS_MODULE
#include "apiHeaderAll.h"

/*
 *	The location of the user proxy certificate within his home directory.
 */
static const char *proxy_file = "/user-proxy";
#endif

/******************************
 *	main gridftp functions
 ******************************/

/*
 * 	Changes the access permissions for a file on a GridFTP server.
 *
 *	@param mode			UNIX like permission set (like "644")
 *	@param gridftpURL	GridFTP URL for this operation
 *	@return				0 on success, 1 on failure
 */
int gridftp_chmod(char *mode, char *gridftpURL);

/*
 *	Requests the checksum of a file on a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL to request file checksum
 *	@return				0 on success, 1 on failure
 */
int gridftp_cksm(char *gridftpURL);

/*
 *	Copies a file from one GridFTP location to another.
 *	GridFTP location.
 *
 *	@param srcGridftpURL	source GridFTP URL to copy from
 *	@param dstGridftpURL	destination GridFTP URL to copy to
 *	@return					0 on success, 1 on failure
 */
int gridftp_cp(char *srcGridftpURL, char *dstGridftpURL);

/*
 *	Deletes a file from a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL for file to delete
 *	@return				0 on success, 1 on failure
 */
int gridftp_del(char *gridftpURL);

/*
 *	Requests the existence of a file on a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL to request file existence
 *	@return				0 on success, 1 on failure
 */
int gridftp_exist(char *gridftpURL);

/*
 *	Requests the supported features of a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL to request supported features
 *	@return				0 on success, 1 on failure
 */
int gridftp_feat(char *gridftpURL);

/*
 *	Gets a file from a GridFTP server to local (iRODS).
 *	This get function is used within the commandline client
 *	and the iRODS GridFTP file driver.
 *
 *	@param srcGridftpURL	source GridFTP URL to get file
 *	@param dstFile			destination file name
 *							(with real path on the filesystem)
 *	@return					0 on success, 1 on failure
 */
int gridftp_get(char *srcGridftpURL, char *dstFile);

/*
 *	Gets a file from a GridFTP server to local iRODS.
 *	This get function is used within iRODS microservice.
 *
 *	@param srcGridftpURL	source GridFTP URL to get file
 *	@param dstFile			destination file name
 *							(with real path on the filesystem)
 *	@return					0 on success, 1 on failure
 */
int gridftp_module_get(char *srcGridftpURL, char *dstIrodsFile);

/*
 *	Does a directory listing on a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL for directory listing
 *	@return				0 on success, 1 on failure
 */
int gridftp_ls(char *gridftpURL);

/*
 *	Creates a directory on a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL of directory to create
 *	@return				0 on success, 1 on failure
 */
int gridftp_mkdir(char *gridftpURL);

/*
 *	Requests the modification date of a file on a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL to request file modification date
 *	@return				0 on success, 1 on failure
 */
int gridftp_modtime(char *gridftpURL);

/*
 *	Moves a file from one GridFTP location to another.
 *
 *	@param srcGridftpURL	source GridFTP URL to move from
 *	@param dstGridftpURL	destination GridFTP URL to move to
 *	@return					0 on success, 1 on failure
 */
int gridftp_mv(char *srcGridftpURL, char *dstGridftpURL);

/*
 *	Puts a file from local (iRODS) to a GridFTP server.
 *	This put function is used within the commandline client
 *	and the iRODS GridFTP file driver.
 *
 *	@param srcFile			source file
 *							(with real path on the filesystem)
 *	@param dstGridftpURL	destination GridFTP URL to put this file
 *	@return					0 on success, 1 on failure
 */
int gridftp_put(char *srcFile, char *dstGridftpURL);

/*
 *	Puts a file from local iRODS to a GridFTP server.
 *	This put function is used within iRODS microservice.
 *
 *	@param srcIrodsFile		source iRODS file
 *	@param dstGridftpURL	destination GridFTP URL to put this file
 *	@return					0 on success, 1 on failure
 */
int gridftp_module_put(char *srcIrodsFile, char *dstGridftpURL);

/*
 *	Deletes a directory on a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL of directory to remove
 *	@return				0 on success, 1 on failure
 */
int gridftp_rmdir(char *gridftpURL);

/*
 *	Requests the size of a file on a GridFTP server.
 *
 *	@param gridftpURL	GridFTP URL to request file size
 *	@return				0 on success, 1 on failure
 */
int gridftp_size(char *gridftpURL);



/***************************
 *	helper functions
 ***************************/
#ifdef GRIDFTP_IRODS_MODULE

/*
 *	Sets the user proxy certificate for use within microservices.
 *
 *	@param rsComm	iRODS server communication handle
 *	@return			0 on success, 1 on failure
 */
int setUserProxy(rsComm_t *rsComm);

#endif

/*
 *	Prints an informational message to stdout/rodsLog, if
 *	Prints GRIDFTP_DEBUG is defined.
 *
 *	@param msg		message to print
 */
void printInfo(char *msg, ...);

/*
 *	Prints an error message to stdout/rodsLog.
 *
 *	@param msg		error message to print
 */
void printErrorMessage(char *msg, ...);

/*
 *	Prints the messages from an error object to stdout/rodsLog.
 *
 *	@param errObject	error message to print
 */
void printErrorObject(globus_object_t *errObject);

/*
 *	Checks the error state of a globus_result_t object returned by
 *	a globus_ftp_client operation. When it contains an error, the
 *	error message is printed to stdout/rodsLog.
 *
 *	@param result	globus_result_t object, that was returned
 *					by a globus_ftp_client operation
 *	@return			1, if the errObject contains an error, otherwise 0
 */
inline int checkResult(globus_result_t result);

/*
 *	Requests whether a single feature is supported by a GridFTP server.
 *
 *	@param handle	an initialized handle
 *	@param url		GridFTP URL of the requested server
 *	@param feature	feature to request
 */
globus_ftp_client_tristate_t gridftp_feat_check(
		globus_ftp_client_handle_t *handle,
		char *url,
		globus_ftp_client_probed_feature_t feature);

#endif /* GRIDFTP_CLIENT_H_ */
