/*
 *	@name		gridftp.h
 *	@author		Christian Loeschen, TU Dresden
 *	@license	BSD
 */

#ifndef GRIDFTP_CLIENT_H_
#define GRIDFTP_CLIENT_H_


/*
 *	iRODS Microservice function that changes the access permissions for
 *	iRODS a file or directory on a GridFTP server.
 *
 *	@param mode	msParam_t containing changed access rights in UNIX style (like "644")
 *	@param url	msParam_t containing GridFTP URL
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpChmod(msParam_t *mode, msParam_t *url, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that requests the checksum of a file on
 *	a GridFTP server.
 *
 *	@param url	msParam_t containing GridFTP file URL
 *	@param out	msParam_t to write requested checksum to
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpCksm(msParam_t *url, msParam_t *out, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that copies a file from one GridFTP
 *	location to another.
 *
 *	@param url	msParam_t containing GridFTP file URL to copy file from
 *	@param out	msParam_t containing GridFTP file URL to copy file to
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpCp(msParam_t *srcUrl, msParam_t *dstUrl, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that deletes a file from a GridFTP server.
 *
 *	@param url	msParam_t containing GridFTP file URL
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpDel(msParam_t *url, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that requests the existence of a file
 *	on a GridFTP server.
 *
 *	@param url	msParam_t containing GridFTP URL
 *	@param out	msParam_t to write result to
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpExist(msParam_t *url, msParam_t *out, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that performs a full module test on a
 *	iRODS GridFTP server.
 *
 *	@param rodsFile	msParam_t containing iRODS file name to use for testing
 *	@param testUrl	msParam_t containing GridFTP directory URL to write test
 *	@param out		msParam_t to write test result to
 *	@param rei		iRODS ruleExecutionInfo structure
 *	@return			1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpFuncTest(msParam_t *rodsFile, msParam_t *testUrl, msParam_t *out, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that gets a file from a GridFTP server
 *	to local iRODS.
 *
 *	@param srcUrl	msParam_t containing GridFTP file URL to get file from
 *	@param rodsFile	msParam_t containing iRODS file name to put file to
 *	@param rei		iRODS ruleExecutionInfo structure
 *	@return			1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpGet(msParam_t *srcUrl, msParam_t *rodsFile, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that does a directory listing on a
 *	GridFTP server.
 *
 *	@param url	msParam_t containing GridFTP directory URL
 *	@param out	msParam_t to write requested directory listing to
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpLs(msParam_t *url, msParam_t *out, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that creates a directory on a GridFTP server.
 *
 *	@param url	msParam_t containing GridFTP directory URL
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpMkdir(msParam_t *url, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that requests the modification date of a
 *	file on a GridFTP server.
 *
 *	@param url	msParam_t containing GridFTP file URL
 *	@param out	msParam_t to write requested modification time to
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpModtime(msParam_t *url, msParam_t *out, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that moves a file from one GridFTP location
 *	to another.
 *
 *	@param url	msParam_t containing GridFTP file URL to move file from
 *	@param out	msParam_t containing GridFTP file URL to move file to
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpMv(msParam_t *srcUrl, msParam_t *dstUrl, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that puts a file from local iRODS to a
 *	GridFTP server.
 *
 *	@param rodsFile	msParam_t containing existing iRODS file name
 *	@param dstUrl	msParam_t containing GridFTP file URL to put file to
 *	@param rei		iRODS ruleExecutionInfo structure
 *	@return			1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpPut(msParam_t *rodsFile, msParam_t *dstUrl, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that deletes a directory on a GridFTP server.
 *
 *	@param url	msParam_t containing GridFTP directory URL
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpRmdir(msParam_t *url, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that requests the size of a file on a
 *	GridFTP server.
 *
 *	@param url	msParam_t containing GridFTP file URL
 *	@param out	msParam_t to write requested size to
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1 on success, on error -344000 (EXEC_CMD_ERROR)
 */
int msiGridftpSize(msParam_t *url, msParam_t *out, ruleExecInfo_t *rei);

/*
 *	iRODS Microservice function that checks the correct module integration.
 *	Does only a little ouput.
 *
 *	@param rei	iRODS ruleExecutionInfo structure
 *	@return		1
 */
int msiGridftpTest(ruleExecInfo_t *rei);

#endif /* GRIDFTP_CLIENT_H_ */
